#!/usr/bin/env python
#
# Copyright (c) 2011 Alon Swartz <alon@turnkeylinux.org>
#
# This file is part of HubTools.
#
# HubTools is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
"""
Launch a new cloud server

Arguments:

    appliance       Appliance name to launch (e.g. core)
    OR
    snapshot-id     Restore snapshot to a new cloud server (e.g., snap-453a051d)

Options:

    --backup-id     TurnKey Backup ID to restore on launch

    --region        Region for instance launch (default: us-east-1)
                    Regions:

                      us-east-1       Virginia (East USA)
                      us-west-1       California (West USA)
                      us-west-2       Oregon (West USA)
                      sa-east-1       Sao Paulo (South America)
                      eu-west-1       Ireland (West Europe)
                      ap-northeast-1  Tokyo (North-East Asia)
                      ap-southeast-1  Singapore (South-East Asia)
                      ap-southeast-2  Sydney (Australia)


    --size          Instance size (default: m1.small)
                    Sizes:

                      t1.micro
                      m1.small
                      m1.medium
                      c1.medium
                      m1.large
                      m1.xlarge
                      c1.xlarge
                      m2.xlarge
                      m2.2xlarge
                      m2.4xlarge
                      hi1.4xlarge

    --arch          Instance architecture <i386|amd64> (default: i386)

    --type          Instance type <s3|ebs> (default: s3)
    --label         Optional server descriptive label

    --root-pass     Root password to set (default: random)
    --db-pass       Database password
    --app-pass      Admin password for application
    --app-email     Admin email address for application
    --app-domain    Domain for application

    --fqdn          Fully qualified domain name to associate
                    e.g., www.tklapp.com. | www.example.com.

Environment variables:

    HUB_APIKEY      Displayed in your Hub account's user profile
"""
import os
import sys
import getopt

from hub import Hub
from hub.formatter import fmt_server_header, fmt_server

def fatal(e):
    print >> sys.stderr, "error: " + str(e)
    sys.exit(1)

def usage(e=None):
    if e:
        print >> sys.stderr, "error: " + str(e)

    print >> sys.stderr, "Syntax: %s <appliance|snapshot-id> [opts]" % (sys.argv[0])
    print >> sys.stderr, __doc__

    sys.exit(1)

def main():
    kwargs = {
        'region': "us-east-1",
        'size': "m1.small",
        'type': "s3",
        'arch': "i386",
        'label': "",
        'root_pass': "",
        'db_pass': "",
        'app_pass': "",
        'app_email': "",
        'app_domain': "",
        'fqdn': "",

        'backup_id': None,
    }
    try:
        opts, args = getopt.gnu_getopt(sys.argv[1:], "h",
                                       ["help" ] +
                                       [  key.replace("_", "-") + "=" for key in kwargs ])
    except getopt.GetoptError, e:
        usage(e)

    for opt, val in opts:
        if opt in ('-h', '--help'):
            usage()

        for kwarg in kwargs:
            if opt == '--' + kwarg.replace('_', '-'):
                kwargs[kwarg] = val
                break

    if len(args) != 1:
        usage("incorrect number of arguments")

    apikey = os.environ.get('HUB_APIKEY')
    if not apikey:
        fatal("HUB_APIKEY not specified in environment")

    name = args[0]
    hub = Hub(apikey)

    try:
        server = hub.servers.launch(name, **kwargs)
    except hub.Error, e:
        if e.name == "Request.MissingArgument":
            arg = e.description.split()[-1]
            fatal("Required argument: --" + arg.replace('_', '-'))
        else:
            fatal(e.description)

    print fmt_server_header()
    print fmt_server(server)

if __name__ == "__main__":
    main()

